/**
 * @Company: 云南奇讯科技有限公司
 * @Author: yxf
 * @Description: 订单列表
 * @Date: 2024-01-16 12:00:38
 */
package logic

import (
	"context"
	"crypto/md5"
	"encoding/hex"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/gogf/gf/v2/container/gset"
	"github.com/gogf/gf/v2/database/gdb"
	"github.com/gogf/gf/v2/encoding/gbase64"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/os/gctx"
	"github.com/gogf/gf/v2/os/gtime"
	"github.com/gogf/gf/v2/util/gconv"
	"github.com/skip2/go-qrcode"
	"github.com/tiger1103/gfast/v3/api/v1/storeApp/order"
	commonService "github.com/tiger1103/gfast/v3/internal/app/common/service"
	storeConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/finance"
	storeCommonConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeCommon"
	storeEventConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeEvent"
	"github.com/tiger1103/gfast/v3/internal/app/store/consts/storeGoods"
	storeMemberConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeMember"
	storeOrderConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeOrder"
	storeFinanceModel "github.com/tiger1103/gfast/v3/internal/app/store/finance/model"
	financeService "github.com/tiger1103/gfast/v3/internal/app/store/finance/service"
	freightService "github.com/tiger1103/gfast/v3/internal/app/store/freight/service"
	goodsService "github.com/tiger1103/gfast/v3/internal/app/store/goods/service"
	storeMemberModel "github.com/tiger1103/gfast/v3/internal/app/store/member/model"
	storeMemberService "github.com/tiger1103/gfast/v3/internal/app/store/member/service"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/dao"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/model"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/model/do"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/service"
	storeCommonModel "github.com/tiger1103/gfast/v3/internal/app/store/storeCommon/model"
	storeCommonService "github.com/tiger1103/gfast/v3/internal/app/store/storeCommon/service"
	"github.com/tiger1103/gfast/v3/internal/app/system/consts"
	systemConsts "github.com/tiger1103/gfast/v3/internal/app/system/consts"
	systemModel "github.com/tiger1103/gfast/v3/internal/app/system/model"
	systemService "github.com/tiger1103/gfast/v3/internal/app/system/service"
	ucenterModel "github.com/tiger1103/gfast/v3/internal/app/ucenter/model"
	ucenterService "github.com/tiger1103/gfast/v3/internal/app/ucenter/service"
	"github.com/tiger1103/gfast/v3/library/liberr"
	"github.com/xuri/excelize/v2"
	"math/rand"
	"strconv"
	"time"
)

func init() {
	service.RegisterStoreOrder(New())
}

func New() service.IStoreOrder {
	return &sStoreOrder{}
}

type sStoreOrder struct{}

func (s *sStoreOrder) List(ctx context.Context, req *model.StoreOrderSearchReq) (listRes *model.StoreOrderSearchRes, err error) {
	listRes = new(model.StoreOrderSearchRes)
	err = g.Try(ctx, func(ctx context.Context) {
		m := dao.StoreOrder.Ctx(ctx).As("a").
			LeftJoin("store_member m", "m.uid=a.uid").
			LeftJoin("ucenter_members u", "u.id=a.uid")
		if req.Uid != "" {
			m = m.Where("a."+dao.StoreOrder.Columns().Uid+" = ?", gconv.Int(req.Uid))
		}
		if req.Status != "" {
			m = m.Where("a."+dao.StoreOrder.Columns().Status+" = ?", gconv.Int(req.Status))
		}
		if req.Keywords != "" {
			switch req.KeywordsType {
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_ORDER_NO:
				m = m.Where("a.order_id like ?", "%"+req.Keywords+"%")
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_REALNAME:
				m = m.Where("m.real_name like ?", "%"+req.Keywords+"%")
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_MOBILE:
				m = m.Where("u.mobile like ?", "%"+req.Keywords+"%")
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_GOODS_NAME:
				m = m.Where("a.order_id in(?)", dao.StoreOrderDetail.Ctx(ctx).Fields("order_id").Where("name like ?", req.Keywords))
			}
		}

		if len(req.DateRange) > 0 {
			m = m.WhereBetween("a.created_at", req.DateRange[0]+" 00:00:00", req.DateRange[1]+" 23:59:59")
		}
		m = m.Group("a.order_id")
		listRes.Total, err = m.CountColumn("a.order_id")
		liberr.ErrIsNil(ctx, err, "获取总行数失败")
		if req.PageNum == 0 {
			req.PageNum = 1
		}
		listRes.CurrentPage = req.PageNum
		if req.PageSize == 0 {
			req.PageSize = consts.PageSize
		}
		sort := "a.created_at desc"
		if req.OrderBy != "" {
			sort = req.OrderBy
		}
		err = m.Fields("a.*").WithAll().Page(req.PageNum, req.PageSize).Order(sort).Scan(&listRes.List)
		liberr.ErrIsNil(ctx, err, "获取数据失败")
	})
	return
}

// Export 导出订单
func (s *sStoreOrder) Export(ctx context.Context, req *model.StoreOrderSearchReq) (exportFileUrl string, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		m := dao.StoreOrder.Ctx(ctx).As("a").
			LeftJoin("store_order_detail detail", "detail.order_id=a.order_id").
			LeftJoin("store_member m", "m.uid=a.uid").
			LeftJoin("ucenter_members u", "u.id=a.uid").
			LeftJoin("sys_dict_data dict", "dict.`dict_type`='store_order_status' AND dict.`dict_value`=a.`status`").
			LeftJoin("sys_dict_data dict2", "dict2.`dict_type`='store_pay_status' AND dict2.`dict_value`=a.`paid`").
			LeftJoin("sys_dict_data dict3", "dict3.`dict_type`='store_order_detail_status' AND dict3.`dict_value`=detail.`status`").
			LeftJoin("sys_dict_data dict4", "dict4.`dict_type`='store_pay_type' AND dict4.`dict_value`=a.`pay_type`").
			LeftJoin("sys_dict_data dict5", "dict5.`dict_type`='store_order_after_sales_status' AND dict5.`dict_value`=detail.`after_sales_status`").
			LeftJoin("store_member_address addr", "addr.id=a.address_id")

		if req.Uid != "" {
			m = m.Where("a."+dao.StoreOrder.Columns().Uid+" = ?", gconv.Int(req.Uid))
		}
		if req.Status != "" {
			m = m.Where("a."+dao.StoreOrder.Columns().Status+" = ?", gconv.Int(req.Status))
		}
		if req.Keywords != "" {
			switch req.KeywordsType {
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_ORDER_NO:
				m = m.Where("a.order_id like ?", "%"+req.Keywords+"%")
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_REALNAME:
				m = m.Where("m.real_name like ?", "%"+req.Keywords+"%")
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_MOBILE:
				m = m.Where("u.mobile like ?", "%"+req.Keywords+"%")
			case storeOrderConsts.STORE_ORDER_SEARCH_KEYWORDS_TYPE_GOODS_NAME:
				m = m.Where("a.order_id in(?)", dao.StoreOrderDetail.Ctx(ctx).Fields("order_id").Where("name like ?", req.Keywords))
			}
		}

		if len(req.DateRange) > 0 {
			m = m.WhereBetween("a.created_at", req.DateRange[0]+" 00:00:00", req.DateRange[1]+" 23:59:59")
		} else {
			liberr.ErrIsNil(ctx, errors.New("请选择时间范围"))
		}

		if req.PageNum == 0 {
			req.PageNum = 1
		}
		// 限制导出pageSize最大值为10000
		req.PageSize = 10000
		sort := "a.created_at desc"
		if req.OrderBy != "" {
			sort = req.OrderBy
		}
		resList, err := m.Fields("a.*", "detail.goods_name as goods_name,", "detail.quantity as quantity,", "detail.real_price as real_price,", "detail.status as detail_status,", "dict.dict_label as order_status_label,", "dict2.dict_label as pay_status_label,", "dict3.dict_label as detail_status_label,", "dict4.dict_label as pay_type_label,", "dict5.dict_label as after_sales_status_label,", "u.nickname as nickname,", "addr.province AS province,", "addr.city AS city,", "addr.district AS district,", "addr.detail AS detail,", "addr.real_name as address_real_name,", "addr.phone as address_phone").
			Page(req.PageNum, req.PageSize).Order(sort).All()
		liberr.ErrIsNil(ctx, err, "获取数据失败")
		if resList == nil || len(resList) == 0 {
			liberr.ErrIsNil(ctx, errors.New("没有数据可以导出"))
		}
		//读取模板
		g.Log().Debug(ctx, g.Cfg().MustGet(ctx, "server.serverRoot").String()+"/export_excel/template/export_order_template.xlsx")
		file, err := excelize.OpenFile(g.Cfg().MustGet(ctx, "server.serverRoot").String() + "/export_excel/template/export_order_template.xlsx")
		liberr.ErrIsNil(ctx, err, "打开模板文件失败")
		num := 0
		sheetName := file.GetSheetName(file.GetActiveSheetIndex())
		for _, item := range resList {
			num++
			rowData := []string{
				item["order_id"].String(),
				item["goods_name"].String(),
				item["quantity"].String(),
				item["real_price"].String(),
				item["detail_status_label"].String(),
				item["after_sales_status_label"].String(),
				item["nickname"].String(),
				item["total_amount"].String(),
				item["pay_amount"].String(),
				item["pay_freight"].String(),
				item["created_at"].String(),
				item["order_status_label"].String(),
				item["pay_status_label"].String(),
				item["pay_time"].String(),
				item["pay_type_label"].String(),
				item["out_trade_no"].String(),
				item["remark"].String(),
				item["buyer_message"].String(),
				item["address"].String() + item["address_real_name"].String() + item["address_phone"].String(),
				item["bonus_points"].String(),
			}
			err = file.SetSheetRow(sheetName, fmt.Sprintf("A%d", num+2), &rowData)
			// 订单信息

		}
		exportFileUrl = fmt.Sprintf("/export_excel/订单导出_%s.xlsx", gtime.Now().Format("YmdHis"))
		path := fmt.Sprintf("%s%s", g.Cfg().MustGet(ctx, "server.serverRoot"), exportFileUrl)
		err = file.SaveAs(path)
		liberr.ErrIsNil(ctx, err, "导出订单失败，保存文件失败")
	})
	return
}

// GetByOrderId 通过订单号获取订单信息
func (s *sStoreOrder) GetByOrderId(ctx context.Context, orderId uint64) (res *model.StoreOrderInfoRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		err = dao.StoreOrder.Ctx(ctx).WithAll().Where(dao.StoreOrder.Columns().OrderId, orderId).Scan(&res)
		res.DiscountAmount = gconv.Float64(fmt.Sprintf("%.2f", res.TotalAmount-res.PayAmount))
		liberr.ErrIsNil(ctx, err, "获取订单信息失败")
	})
	return
}

// GetByOutTradeNo 通过交易订单号获取订单信息
func (s *sStoreOrder) GetByOutTradeNo(ctx context.Context, outTradeNo interface{}) (res *model.StoreOrderInfoRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		err = dao.StoreOrder.Ctx(ctx).WithAll().Where(dao.StoreOrder.Columns().OutTradeNo, outTradeNo).Scan(&res)
		liberr.ErrIsNil(ctx, err, "获取订单信息失败")
	})
	return
}

// Edit 后台修改订单数据
func (s *sStoreOrder) Edit(ctx context.Context, req *do.StoreOrder) (err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		_, err = dao.StoreOrder.Ctx(ctx).Where(dao.StoreOrder.Columns().OrderId, req.OrderId).Update(req)
		liberr.ErrIsNil(ctx, err, "修改订单失败")
	})
	return
}

// Delete 删除订单数据
func (s *sStoreOrder) Delete(ctx context.Context, orderIds []string) (err error) {
	if orderIds != nil && len(orderIds) > 0 {
		return g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
			return g.Try(ctx, func(ctx context.Context) {
				_, err = dao.StoreOrder.Ctx(ctx).Delete(dao.StoreOrder.Columns().OrderId+" in (?)", orderIds)
				liberr.ErrIsNil(ctx, err, "删除订单失败")
				_, err = dao.StoreOrderDetail.Ctx(ctx).Delete(dao.StoreOrder.Columns().OrderId+" in (?)", orderIds)
				liberr.ErrIsNil(ctx, err, "删除订单明细失败")
				_, err = dao.StoreOrderLog.Ctx(ctx).Delete(dao.StoreOrder.Columns().OrderId+" in (?)", orderIds)
				liberr.ErrIsNil(ctx, err, "删除订单日志失败")
				_, err = dao.StoreOrderAfterSales.Ctx(ctx).Delete(dao.StoreOrder.Columns().OrderId+" in (?)", orderIds)
				liberr.ErrIsNil(ctx, err, "删除订单售后信息失败")
			})
		})
	}
	// 删除订单相关数据
	return
}

// FilterCreateBefore 创建订单之前检查
func (s *sStoreOrder) FilterCreateBefore(ctx context.Context, req *order.StoreOrderCreateReq,
	memberInfo *storeMemberModel.StoreMemberInfoRes) (pass bool, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		var jsonBytes []byte
		jsonBytes, err = json.Marshal(req)
		if err != nil {
			return
		}
		// 计算请求内容的MD5哈希值
		hasher := md5.New()
		hasher.Write(jsonBytes)
		hashValue := hex.EncodeToString(hasher.Sum(nil))

		// 构造缓存key，包含用户ID和请求内容的hash值
		cacheKey := fmt.Sprintf("order_create_%d_%s", memberInfo.Uid, hashValue)
		cache := commonService.Cache()
		// 检查是否存在相同的订单创建请求
		if cache.Get(ctx, cacheKey).String() == "1" {
			pass = false
			return
		}
		// 设置缓存，10秒内相同的请求不允许重复提交
		cache.Set(ctx, cacheKey, 1, 10*time.Second)
		pass = true
	})
	return
}

// Create 创建订单
func (s *sStoreOrder) Create(ctx context.Context, memberInfo *storeMemberModel.StoreMemberInfoRes, req *order.StoreOrderCreateReq) (res *order.StoreOrderCreateRes, err error) {
	// 生成订单ID
	// todo 创建订单时加入选择配送方式，快递和到店自提
	orderId, err := commonService.SnowID().GenID()
	liberr.ErrIsNil(ctx, err, "生成订单ID失败")
	err = g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
		return g.Try(ctx, func(ctx context.Context) {
			// 检查收货地址是否有效
			addressInfo, err := storeMemberService.StoreMemberAddress().GetById(ctx, req.AddressId)
			if err != nil || addressInfo == nil {
				liberr.ErrIsNil(ctx, errors.New("收货地址无效"))
			}
			// 生成订单基本信息
			orderAddDo := &do.StoreOrder{
				Uid:          memberInfo.Uid,
				OrderId:      gconv.Uint64(orderId),
				OutTradeNo:   gconv.Uint64(orderId),
				Remark:       req.Remark,
				BuyerMessage: req.BuyerMessage,
				AddressId:    req.AddressId,
				OrderType:    req.OrderType,
			}
			var totalAmount float64 = 0
			// 处理订单明细信息
			// 判断扣库存的时机
			reduceStockTime := gconv.Int(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "reduce_stock_time"))

			// 获取商品信息
			var orderDetailAddReqList []*model.StoreOrderDetailAddReq
			// 创建订单商品明细信息
			for _, orderGoods := range req.GoodsList {
				goodsAttrValue, err := goodsService.StoreGoodsAttrValue().GetById(ctx, orderGoods.GoodsAttr)
				liberr.ErrIsNil(ctx, err, "获取商品属性信息失败")
				if goodsAttrValue == nil || goodsAttrValue.Goods == nil {
					liberr.ErrIsNil(ctx, errors.New("商品属性信息无效"))
				}
				if goodsAttrValue.Goods.IsShow != 1 {
					liberr.ErrIsNil(ctx, errors.New(fmt.Sprintf("商品：%s 已下架", goodsAttrValue.Goods.Name)))
				}

				// 扣除库存
				if reduceStockTime == storeCommonConsts.STORE_CONFIG_REDUCTION_STOCK_TIME_CREATE_ORDER {
					if goodsAttrValue.VirtualType == storeGoods.STORE_GOODS_COUPON_GOODS {
						err = storeCommonService.StoreCoupon().JudgeCouponSend(ctx, goodsAttrValue.CouponId, int(orderGoods.Quantity))
						liberr.ErrIsNil(ctx, err)
					}

					err = goodsService.StoreGoodsAttrValue().ReduceStock(ctx, goodsAttrValue.Id, orderGoods.Quantity)
					liberr.ErrIsNil(ctx, err)
				}

				// 生成订单明细ID
				orderDetailSnowId, err := commonService.SnowID().GenID()
				liberr.ErrIsNil(ctx, err, "生成订单明细ID失败")
				tmpOrderDetailAddReq := &model.StoreOrderDetailAddReq{
					Id:            orderDetailSnowId,
					Uid:           memberInfo.Uid, // 用户ID
					OrderId:       orderId,
					GoodsId:       goodsAttrValue.GoodsId,                                                                                  // 商品ID
					GoodsAttr:     goodsAttrValue.Id,                                                                                       // 商品属性ID
					GoodsName:     goodsAttrValue.Goods.Name,                                                                               // 商品名称
					GoodsImg:      goodsAttrValue.Goods.Thumb,                                                                              // 商品属性缩略图
					GoodsUnit:     goodsAttrValue.Goods.UnitName,                                                                           // 商品单位
					GoodsAttrName: goodsAttrValue.Suk,                                                                                      // 商品属性名称
					GoodsPrice:    goodsAttrValue.Price,                                                                                    // 商品价格
					CartId:        orderGoods.CartId,                                                                                       // 购物车ID
					Quantity:      orderGoods.Quantity,                                                                                     // 购买数量
					RealPrice:     goodsService.StoreGoodsAttrValue().CalculatePrice(ctx, memberInfo, goodsAttrValue, orderGoods.Quantity), // 商品实际价格
					GoodsCategory: goodsAttrValue.Goods.CateId,                                                                             // 商品分类ID
				}

				// 修改订单总额 = 商品实际价格 * 数量
				totalAmount += gconv.Float64(tmpOrderDetailAddReq.RealPrice) * gconv.Float64(orderGoods.Quantity)
				orderDetailAddReqList = append(orderDetailAddReqList, tmpOrderDetailAddReq)
			}
			// 保存订单基本金额
			orderAddDo.TotalAmount = totalAmount
			// todo 扣减信息计算包含积分，优惠券，满减，满折，会员折扣，首单等============================================

			// 使用优惠券
			if len(req.Coupons) > 0 {
				couponSet := gset.NewStrSetFrom(req.Coupons, true)
				if couponSet.Size() > 0 {
					for _, couponNumber := range couponSet.Slice() {
						var couponInfo *storeMemberModel.StoreMemberCouponInfoRes
						couponInfo, err = storeMemberService.StoreMemberCoupon().GetByCouponNumber(ctx, couponNumber, memberInfo.Uid)
						if err != nil || couponInfo == nil {
							liberr.ErrIsNil(ctx, errors.New("优惠券不存在或无效"))
						}
						// 是否允许叠加使用
						if couponSet.Size() > 1 && couponInfo.Coupon.AllowStack == 0 {
							liberr.ErrIsNil(ctx, errors.New(fmt.Sprintf("优惠券%s不允许叠加使用", couponInfo.CouponNumber)))
						}
						err = storeMemberService.StoreMemberCoupon().IsValid(ctx, couponInfo, totalAmount)
						liberr.ErrIsNil(ctx, err)
						// 计算优惠券抵扣，这里使用的是订单的总金额来进行计算
						couponDiscountAmount, err := service.StoreOrderDiscount().CalculateCouponDiscount(ctx, couponInfo, orderAddDo.TotalAmount.(float64), orderDetailAddReqList)
						liberr.ErrIsNil(ctx, err)
						if couponDiscountAmount > 0 {
							totalAmount -= couponDiscountAmount
							// 如果优惠券抵扣后金额小于0，则总金额为0
							if totalAmount < 0 {
								totalAmount = 0
							}
							// 更新优惠券使用记录
							err = storeMemberService.StoreMemberCoupon().ChangeStatus(ctx, couponNumber, storeMemberConsts.STORE_MEMBER_COUPON_STATUS_USED, "，使用订单号："+gconv.String(orderId))
							liberr.ErrIsNil(ctx, err, "更新优惠券使用记录失败")
							// 保存订单使用的优惠券记录
							err = service.StoreOrderDiscount().Add(ctx, &model.StoreOrderDiscountAddReq{
								OrderId:        orderId,
								DiscountType:   storeOrderConsts.STORE_ORDER_DISCOUNT_TYPE_COUPON,
								DiscountAmount: couponDiscountAmount,
								DiscountExtra:  couponNumber,
								DiscountRemark: fmt.Sprintf("优惠券号：%s", couponNumber),
								DiscountName:   "优惠券抵扣",
							})
							liberr.ErrIsNil(ctx, err, "保存订单使用的积分记录失败")
						}
					}
				}
			}

			if req.Points > 0 {
				if gconv.Int(req.Points) > memberInfo.Points {
					liberr.ErrIsNil(ctx, errors.New("积分不足"))
				}
				pointsDiscountAmount, usedPoints, err := storeMemberService.StoreMemberPoints().CalculatePointsDiscount(ctx, req.Points, totalAmount)
				liberr.ErrIsNil(ctx, err)
				totalAmount -= pointsDiscountAmount
				// 减去积分
				memberInfo.Points -= gconv.Int(usedPoints)
				err = storeMemberService.StoreMember().UpdatePoints(ctx, memberInfo.Uid, memberInfo.Points)
				liberr.ErrIsNil(ctx, err, "更新用户积分失败")
				// 记录用户积分使用记录
				err = storeMemberService.StoreMemberPoints().Add(ctx, &storeMemberModel.StoreMemberPointsAddReq{
					Uid:        memberInfo.Uid,
					Title:      "使用积分购物",
					Number:     0 - gconv.Int(usedPoints),
					Balance:    memberInfo.Points,
					Remark:     "订单：" + gconv.String(orderId),
					PointsType: storeMemberConsts.STORE_MEMBER_POINTS_TYPE_SUB,
				})
				liberr.ErrIsNil(ctx, err, "记录积分使用记录失败")
				// 保存订单使用的积分记录
				err = service.StoreOrderDiscount().Add(ctx, &model.StoreOrderDiscountAddReq{
					OrderId:        orderId,
					DiscountType:   storeOrderConsts.STORE_ORDER_DISCOUNT_TYPE_POINTS,
					DiscountAmount: pointsDiscountAmount,
					DiscountExtra:  gconv.String(usedPoints),
					DiscountRemark: fmt.Sprintf("使用积分：%d", usedPoints),
					DiscountName:   "积分抵扣",
				})
				liberr.ErrIsNil(ctx, err, "保存订单使用的积分记录失败")
			}
			// 计算扣减信息结束=========================================================================================

			// 计算运费
			orderAddDo.FreightPrice, err = freightService.StoreFreightTemplate().Freight(ctx, req.AddressId, req.GoodsList)
			liberr.ErrIsNil(ctx, err, "计算运费失败")

			orderAddDo.PayFreight = orderAddDo.FreightPrice
			// 应付订单总额
			orderAddDo.PayAmount = totalAmount
			// 获取积分兑换比率，计算赠送的积分
			bonusPointsRate := gconv.Float64(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "bonus_points"))
			if bonusPointsRate > 0 {
				orderAddDo.BonusPoints = gconv.Int(gconv.Float64(orderAddDo.PayAmount) * bonusPointsRate)
			}
			// 保存订单主表信息
			_, err = dao.StoreOrder.Ctx(ctx).Insert(orderAddDo)
			liberr.ErrIsNil(ctx, err, "保存订单主表信息失败")
			// 保存订单明细表信息
			_, err = dao.StoreOrderDetail.Ctx(ctx).Data(orderDetailAddReqList).Insert()
			liberr.ErrIsNil(ctx, err, "保存订单明细表信息失败")
			// 删除购物车中的商品条目
			cartIds := make([]uint64, 0)
			for _, orderDetail := range orderDetailAddReqList {
				if orderDetail.CartId > 0 {
					cartIds = append(cartIds, orderDetail.CartId)
				}
			}
			if len(cartIds) > 0 {
				err = storeCommonService.StoreCart().Delete(ctx, cartIds)
				liberr.ErrIsNil(ctx, err, "删除购物车中的商品条目失败")
			}
			commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_ORDER_CREATED, orderId)
		})
	})
	if err == nil {
		// 没有出错，保存订单ID到返回结果中
		res = new(order.StoreOrderCreateRes)
		res.OrderId = orderId
	}
	return
}

// Pay 支付订单
func (s *sStoreOrder) Pay(ctx context.Context, payArgs *model.StoreOrderPayArgs) (res interface{}, err error) {
	// 微信支付返回的res是jsSdkConfig信息，用于前端调起支付
	// 支付后需要发送支付成功事件来处理订单状态，因为第三方平台是异步的，所以前端需要轮询订单状态
	var orderInfo *model.StoreOrderInfoRes
	err = g.Try(ctx, func(ctx context.Context) {
		// 检查订单是否有效
		err = dao.StoreOrder.Ctx(ctx).WithAll().Where(g.Map{
			"uid":      payArgs.Uid,
			"order_id": payArgs.OrderId,
			"status":   storeOrderConsts.STORE_ORDER_STATUS_UNPAID,
		}).Scan(&orderInfo)
		if err != nil || orderInfo == nil {
			liberr.ErrIsNil(ctx, errors.New("订单信息无效"))
		}

		// 非线下支付的订单，检查是否超时
		if payArgs.PayType != storeOrderConsts.STORE_PAYTYPE_OFFLINE_PAY {
			cancelOrderTime := orderInfo.CreatedAt.Add(gconv.Duration(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "cancel_order_time")) * time.Minute)
			if gtime.Now().Timestamp() > cancelOrderTime.Timestamp() {
				// 订单超时，取消订单
				dao.StoreOrder.Ctx(ctx).Where(dao.StoreOrder.Columns().OrderId, payArgs.OrderId).Update(&do.StoreOrder{
					Status: storeOrderConsts.STORE_ORDER_STATUS_CLOSED,
				})
				service.StoreOrderLog().Add(ctx, &do.StoreOrderLog{
					OrderId: orderInfo.OrderId,
					LogType: storeOrderConsts.STORE_ORDER_LOG_TYPE_CANCEL,
					Content: "订单超时，系统自动取消订单",
				})
				liberr.ErrIsNil(ctx, errors.New("订单已超时，请重新下单"))
			}
		}

		orderInfo.PayType = payArgs.PayType
		// 生成交易订单号，防止第三方平台支付时out_trade_no重复问题
		orderInfo.OutTradeNo, err = commonService.SnowID().GenID()
		_, err = dao.StoreOrder.Ctx(ctx).Where(dao.StoreOrder.Columns().OrderId, payArgs.OrderId).Update(&do.StoreOrder{
			OutTradeNo: orderInfo.OutTradeNo,
		})
		liberr.ErrIsNil(ctx, err, "生成交易订单号失败")
		switch payArgs.PayType {
		case storeOrderConsts.STORE_PAYTYPE_WECHAT_PAY:
			res, err = service.StorePayment().WechatPay(ctx, payArgs.Openid, payArgs.WechatPayType, orderInfo)
		case storeOrderConsts.STORE_PAYTYPE_ALIPAY:
			err = service.StorePayment().Alipay(ctx, orderInfo)
		case storeOrderConsts.STORE_PAYTYPE_BALANCE_PAY:
			err = service.StorePayment().BalancePay(ctx, orderInfo)
		case storeOrderConsts.STORE_PAYTYPE_OFFLINE_PAY:
			err = service.StorePayment().OfflinePay(ctx, orderInfo)
		case storeOrderConsts.STORE_PAYTYPE_WECHAT_FRIEND_PAY:
			res, err = service.StorePayment().WechatFriendPay(ctx, payArgs.Openid, orderInfo)
		case storeOrderConsts.STORE_PAYTYPE_WECHAT_NATIVE:
			var codeUrl string
			codeUrl, err = service.StorePayment().WechatNativePay(ctx, orderInfo)
			if err == nil {
				// 微信扫码支付，需要返回二维码信息
				qrcodeObj, err := qrcode.New(codeUrl, qrcode.Medium)
				if err == nil {
					qrcodeBytes, err := qrcodeObj.PNG(256)
					if err == nil {
						// 这里返回的是base64编码的二维码图片，前端可以直接显示，需要加上前缀data:image/png;base64,
						res = gbase64.EncodeToString(qrcodeBytes)
					}
				}
			}
		}
		liberr.ErrIsNil(ctx, err)
	})
	return
}

// ConfirmPayment 手动确认支付
func (s *sStoreOrder) ConfirmPayment(ctx context.Context, orderId uint64) (err error) {
	return g.Try(ctx, func(ctx context.Context) {
		var orderInfo *model.StoreOrderInfoRes
		err = dao.StoreOrder.Ctx(ctx).Where("order_id", orderId).Scan(&orderInfo)
		liberr.ErrIsNil(ctx, err, "获取订单信息失败")
		if orderInfo == nil {
			liberr.ErrIsNil(ctx, errors.New("订单信息无效"), "订单信息无效")
		}
		if orderInfo.Status != storeOrderConsts.STORE_ORDER_STATUS_UNPAID {
			liberr.ErrIsNil(ctx, errors.New("订单状态错误"), "订单状态错误")
		}
		liberr.ErrIsNil(ctx, err, "更新订单商品信息失败")
		// 更新订单支付状态
		eventData := &storeCommonModel.StorePaymentEventData{
			OutTradeNo: gconv.String(orderInfo.OutTradeNo),
			PayType:    storeOrderConsts.STORE_PAYTYPE_MANUAL_CONFIRM_PAY,
			Message:    "订单支付成功",
			Attach:     "order",
			Paid:       storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS,
			Amount:     orderInfo.PayAmount + orderInfo.PayFreight,
		}
		commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_PAYMENT_STATUS_CHANGE, ctx, eventData)
	})
}

// Cancel 取消订单
func (s *sStoreOrder) Cancel(ctx context.Context, orderId uint64) (err error) {
	return g.Try(ctx, func(ctx context.Context) {
		var orderInfo *model.StoreOrderInfoRes
		orderInfo, err = s.GetByOrderId(ctx, orderId)
		liberr.ErrIsNil(ctx, err, "获取订单信息失败")
		if orderInfo == nil {
			liberr.ErrIsNil(ctx, errors.New("订单信息无效"))
		}
		if orderInfo.Paid == storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS {
			liberr.ErrIsNil(ctx, errors.New("订单已支付完成，请使用售后退款退货"))
		}
		err = s.Edit(ctx, &do.StoreOrder{
			OrderId: orderId,
			Status:  storeOrderConsts.STORE_ORDER_STATUS_CANCELED,
		})
		liberr.ErrIsNil(ctx, err, "修改订单状态失败")
		// 返还优惠券和积分
		err = service.StoreOrderDiscount().Return(ctx, orderId)
		liberr.ErrIsNil(ctx, err)
	})
}

// Deliver 订单发货
func (s *sStoreOrder) Deliver(ctx context.Context, req *model.StoreOrderDetailDeliveryReq) (err error) {
	return g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
		return g.Try(ctx, func(ctx context.Context) {
			var orderInfo *model.StoreOrderInfoRes
			// 获取订单信息
			orderInfo, err = s.GetByOrderId(ctx, req.OrderId)
			liberr.ErrIsNil(ctx, err)
			if orderInfo == nil || orderInfo.Status != storeOrderConsts.STORE_ORDER_STATUS_WAITDELIVER {
				liberr.ErrIsNil(ctx, errors.New("订单信息无效"))
			}
			switch req.DeliveryType {
			case storeCommonConsts.STORE_DELIVERY_TYPE_HOME_DELIVERY:
				if req.Deliveryman == 0 {
					liberr.ErrIsNil(ctx, errors.New("请选择配送员"))
				}
				req.ExpressCompany = 0
				req.ExpressNo = ""
			case storeCommonConsts.STORE_DELIVERY_TYPE_EXPRESS:
				if req.ExpressCompany == 0 {
					liberr.ErrIsNil(ctx, errors.New("请选择快递公司"))
				}
				if req.ExpressNo == "" {
					liberr.ErrIsNil(ctx, errors.New("请输入快递单号"))
				}
				req.Deliveryman = 0
			default:
				req.Deliveryman = 0
				req.ExpressCompany = 0
				req.ExpressNo = ""
			}
			// 获取自动收货时间
			autoConfirmDay := gconv.Int(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "auto_confirm_day"))
			//确定发货方式(字典)
			/*deliveryTypeDict := new(system.GetDictRes)
			deliveryTypeDict, err = commonService.SysDictData().GetDictWithDataByType(ctx, "", strconv.Itoa(req.DeliveryType))*/
			//常量判断送货方式
			var deliveryType string
			switch req.DeliveryType {
			case storeCommonConsts.STORE_DELIVERY_TYPE_EXPRESS:
				deliveryType = "快递配送"
				break
			case storeCommonConsts.STORE_DELIVERY_TYPE_HOME_DELIVERY:
				deliveryType = "送货上门"
				break
			case storeCommonConsts.STORE_DELIVERY_TYPE_SELF_PICKUP:
				deliveryType = "自提"
				break
			case storeCommonConsts.STORE_DELIVERY_TYPE_NO_DELIVERY:
				deliveryType = "无需配送"
				break
			case storeCommonConsts.STORE_DELIVERY_TYPE_AUTO_DELIVERY:
				deliveryType = "自动发货"
				break
			default:
				deliveryType = "无法确定送货方式"
			}
			if req.IsSplitDeliver {
				// 拆分发货
				for _, delivery := range req.DeliveryList {
					var orderDetailInfo *model.StoreOrderDetailInfoRes
					err = dao.StoreOrderDetail.Ctx(ctx).WherePri(delivery.Id).Scan(&orderDetailInfo)
					liberr.ErrIsNil(ctx, err, "获取订单明细信息失败")
					// 检查数量
					unDeliveredNum := orderDetailInfo.Quantity - orderDetailInfo.Delivered
					if delivery.CurrentDeliverNum > unDeliveredNum {
						liberr.ErrIsNil(ctx, errors.New("发货数量超过未发货数量"))
					}
					// 是否虚拟商品
					goodsAttrInfo, err := goodsService.StoreGoodsAttrValue().GetById(ctx, orderDetailInfo.GoodsAttr)
					liberr.ErrIsNil(ctx, err, "获取商品属性信息失败")
					id, err := commonService.SnowID().GenID()
					liberr.ErrIsNil(ctx, err, "生成ID失败")
					storeDeliverData := &do.StoreDelivery{
						Id:            id,
						OrderDetailId: delivery.Id,
						OrderId:       req.OrderId,
						Deliveryman:   req.Deliveryman,
						Remark:        req.Remark,
						DeliveryNum:   delivery.CurrentDeliverNum,
						UserId:        systemService.Context().GetUserId(ctx),
					}
					if goodsAttrInfo.Goods.GoodsType == storeGoods.STORE_GOODS_NOMAL_GOODS {
						storeDeliverData.DeliveryType = req.DeliveryType
						storeDeliverData.ExpressCompany = req.ExpressCompany
						storeDeliverData.ExpressNo = req.ExpressNo
					}
					if goodsAttrInfo.Goods.GoodsType == storeGoods.STORE_GOODS_CARDPASSWORD_GOODS {
						// 卡密商品发货
						err = goodsService.StoreGoodsVirtual().Deliver(ctx, orderDetailInfo.GoodsAttr, delivery.CurrentDeliverNum, orderInfo.OrderId, orderInfo.Uid)
						liberr.ErrIsNil(ctx, err)
						storeDeliverData.Remark = req.Remark + " 卡密商品发货"
					}
					// 优惠券发货
					if goodsAttrInfo.Goods.GoodsType == storeGoods.STORE_GOODS_COUPON_GOODS {
						err = storeCommonService.StoreCoupon().SendCouponMember(ctx, &storeCommonModel.SendCouponMemeberReq{
							MemberIds: []uint64{orderInfo.Uid},
							CouponId:  goodsAttrInfo.CouponId,
						}, "手动发货")
						liberr.ErrIsNil(ctx, err)
					}
					_, err = dao.StoreDelivery.Ctx(ctx).Insert(storeDeliverData)
					liberr.ErrIsNil(ctx, err)
					// 更新订单明细状态和自动收货时间
					data := &do.StoreOrderDetail{
						Delivered:            orderDetailInfo.Delivered + delivery.CurrentDeliverNum,
						AutoConfirmReceiptAt: gtime.Now().AddDate(0, 0, autoConfirmDay),
					}
					// 订单明细已全部发货完成,将状态修改为待收货
					if orderDetailInfo.Quantity == data.Delivered {
						data.Status = storeOrderConsts.STORE_ORDER_DETAIL_STATUS_WAITRECEIVING
					}
					_, err = dao.StoreOrderDetail.Ctx(ctx).WherePri(delivery.Id).Update(data)
					liberr.ErrIsNil(ctx, err)

					// 发送通知给用户
					err = storeMemberService.StoreMemberNotification().Add(ctx, &storeMemberModel.StoreMemberNotificationAddReq{
						Uid:   orderInfo.Uid,
						Title: "商品已发货",
						//Content: `{"orderId":"` + gconv.String(req.OrderId) + `","goodsName":"` + orderDetailInfo.GoodsName + `","goodsNum":"` + gconv.String(delivery.CurrentDeliverNum) + `"}`,
						Content: `订单编号：` + gconv.String(req.OrderId) + `<br/>送货方式：` + deliveryType + `<br/>商品名称：` + orderDetailInfo.GoodsName + `<br/>数量：` + gconv.String(delivery.CurrentDeliverNum),
						Type:    storeMemberConsts.STORE_MEMBER_NOTIFICATION_TYPE_LOGISTICS,
						Author:  "系统通知",
					})
					liberr.ErrIsNil(ctx, err)
				}
			} else {
				// 正常发货
				for _, orderGoodsDetail := range orderInfo.OrderGoodsDetail {
					// 发货数量
					var deliveryNum = orderGoodsDetail.Quantity - orderGoodsDetail.Delivered
					if deliveryNum > 0 {
						// 是否虚拟商品
						goodsAttrInfo, err := goodsService.StoreGoodsAttrValue().GetById(ctx, orderGoodsDetail.GoodsAttr)
						liberr.ErrIsNil(ctx, err, "获取商品属性信息失败")
						id, err := commonService.SnowID().GenID()
						liberr.ErrIsNil(ctx, err, "生成ID失败")
						storeDeliverData := &do.StoreDelivery{
							Id:             id,
							OrderDetailId:  orderGoodsDetail.Id,
							OrderId:        req.OrderId,
							DeliveryType:   req.DeliveryType,
							ExpressCompany: req.ExpressCompany,
							ExpressNo:      req.ExpressNo,
							Deliveryman:    req.Deliveryman,
							Remark:         req.Remark,
							DeliveryNum:    deliveryNum,
							UserId:         systemService.Context().GetUserId(ctx),
						}
						// 卡密商品发货
						if goodsAttrInfo.Goods.GoodsType == storeGoods.STORE_GOODS_CARDPASSWORD_GOODS {
							err = goodsService.StoreGoodsVirtual().Deliver(ctx, orderGoodsDetail.GoodsAttr, deliveryNum, orderInfo.OrderId, orderInfo.Uid)
							liberr.ErrIsNil(ctx, err)
							storeDeliverData.Remark = req.Remark + " 卡密商品发货"
						}
						// 优惠券发货
						if goodsAttrInfo.Goods.GoodsType == storeGoods.STORE_GOODS_COUPON_GOODS {
							err = storeCommonService.StoreCoupon().SendCouponMember(ctx, &storeCommonModel.SendCouponMemeberReq{
								MemberIds: []uint64{orderInfo.Uid},
								CouponId:  goodsAttrInfo.CouponId,
							}, "手动发货")
							liberr.ErrIsNil(ctx, err)
						}
						_, err = dao.StoreDelivery.Ctx(ctx).Insert(storeDeliverData)
						liberr.ErrIsNil(ctx, err)
						// 更新订单明细状态
						_, err = dao.StoreOrderDetail.Ctx(ctx).WherePri(orderGoodsDetail.Id).Update(&do.StoreOrderDetail{
							Delivered:            orderGoodsDetail.Quantity,
							Status:               storeOrderConsts.STORE_ORDER_DETAIL_STATUS_WAITRECEIVING,
							AutoConfirmReceiptAt: gtime.Now().AddDate(0, 0, autoConfirmDay),
						})
						liberr.ErrIsNil(ctx, err)
						// 发送通知给用户
						err = storeMemberService.StoreMemberNotification().Add(ctx, &storeMemberModel.StoreMemberNotificationAddReq{
							Uid:   orderInfo.Uid,
							Title: "商品已发货",
							//Content: `{"orderId":"` + gconv.String(req.OrderId) + `","goodsName":"` + orderGoodsDetail.GoodsName + `","goodsNum":"` + gconv.String(deliveryNum) + `"}`,
							Content: `订单编号：` + gconv.String(req.OrderId) + `<br/>送货方式：` + deliveryType + `<br/>商品名称：` + orderGoodsDetail.GoodsName + `<br/>数量：` + gconv.String(deliveryNum),
							Type:    storeMemberConsts.STORE_MEMBER_NOTIFICATION_TYPE_LOGISTICS,
							Author:  "系统通知",
						})
						liberr.ErrIsNil(ctx, err)
					}
				}
			}
			liberr.ErrIsNil(ctx, err)
			// 检查订单明细未发货数量,更新订单状态
			// 获取订单信息
			err = dao.StoreOrder.Ctx(ctx).WherePri(req.OrderId).Scan(&orderInfo)
			liberr.ErrIsNil(ctx, err, "获取订单信息失败")
			if orderInfo == nil {
				liberr.ErrIsNil(ctx, errors.New("订单信息无效"))
			}
			unDeliveredCount, err := dao.StoreOrderDetail.Ctx(ctx).Where("order_id", req.OrderId).Where("quantity <> delivered").Count()
			liberr.ErrIsNil(ctx, err, "获取订单未发货数量失败")
			if unDeliveredCount == 0 {
				// 已经全部发货完成则更新订单状态为已发货
				_, err = dao.StoreOrder.Ctx(ctx).Where("order_id", req.OrderId).Update(&do.StoreOrder{
					Status: storeOrderConsts.STORE_ORDER_STATUS_DELIVERED,
				})
			}
			commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_ORDER_DETAIL_DELIVERED, req.OrderId)
		})
	})
}

// Verify 使用核销码对订单进行核销
func (s *sStoreOrder) Verify(ctx context.Context, req *model.StoreOrderVerifyReq) (err error) {
	return g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
		return g.Try(ctx, func(ctx context.Context) {
			var orderInfo *model.StoreOrderInfoRes
			err = dao.StoreOrder.Ctx(ctx).Where("order_id", req.OrderId).Scan(&orderInfo)
			if err != nil || orderInfo == nil {
				liberr.ErrIsNil(ctx, errors.New("订单信息无效"))
			}
			if orderInfo.VerifyCode != "" {
				liberr.ErrIsNil(ctx, errors.New("订单已核销"))
			}
			_, err = dao.StoreOrder.Ctx(ctx).Where("order_id", req.OrderId).Update(&do.StoreOrder{
				Status:     storeOrderConsts.STORE_ORDER_STATUS_COMPLETED,
				VerifyCode: req.VerifyCode,
			})
			liberr.ErrIsNil(ctx, err, "更新订单状态失败")
		})
	})
}

func (s *sStoreOrder) Complete(ctx context.Context, orderId uint64) (err error) {
	// 处理订单完结的后续操作
	// 订单日志记录
	// 发送订单完成通知，其它模块监听该事件进行处理，用户积分增加，返佣等
	return g.Try(ctx, func(ctx context.Context) {
		_, err = dao.StoreOrder.Ctx(ctx).Where("order_id", orderId).Update(&do.StoreOrder{
			Status: storeOrderConsts.STORE_ORDER_STATUS_COMPLETED,
		})
		liberr.ErrIsNil(ctx, err, "更新订单状态失败")
		err = service.StoreOrderLog().Add(ctx, &do.StoreOrderLog{
			OrderId: orderId,
			Content: "订单已完成",
			LogType: storeOrderConsts.STORE_ORDER_LOG_TYPE_COMPLETED,
		})
		liberr.ErrIsNil(ctx, err)
		commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_ORDER_COMPLETED, ctx, orderId)
	})
}

// CheckOrderAuthByMemberId 检查订单是否可以操作
func (s *sStoreOrder) CheckOrderAuthByMemberId(ctx context.Context, orderId uint64) (memberInfo *ucenterModel.UCenterMembersInfoRes, orderInfo *model.StoreOrderInfoRes, err error) {
	memberInfo = ucenterService.Context().GetLoginMember(ctx)
	orderInfo, err = service.StoreOrder().GetByOrderId(ctx, orderId)
	if err != nil || orderInfo == nil {
		return nil, nil, errors.New("订单不存在")
	}
	if orderInfo.Uid != memberInfo.Id {
		return nil, nil, errors.New("没有操作权限")
	}
	return
}

// HandlePayStatus 处理订单支付结果,这里可能是支付宝回调,微信回调,或者其他支付渠道的回调
func (s *sStoreOrder) HandlePayStatus(ctx context.Context, data *storeCommonModel.StorePaymentEventData) (err error) {
	return g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
		return g.Try(ctx, func(ctx context.Context) {
			orderInfo, err := service.StoreOrder().GetByOutTradeNo(ctx, data.OutTradeNo)
			liberr.ErrIsNil(ctx, err)
			if orderInfo == nil {
				liberr.ErrIsNil(ctx, errors.New("订单不存在"))
			}
			// 订单更新数据
			orderDo := &do.StoreOrder{
				Paid:    data.Paid,
				PayTime: gtime.Now(),
				PayType: data.PayType,
				PayNo:   data.OutTradeNo,
			}
			if orderDo.Paid == storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS {
				// 更新订单状态为待发货
				orderDo.Status = storeOrderConsts.STORE_ORDER_STATUS_WAITDELIVER
				// 更新订单明细商品状态为待发货
				_, err = dao.StoreOrderDetail.Ctx(ctx).Where(dao.StoreOrderDetail.Columns().OrderId, orderInfo.OrderId).Update(orderDo)
				liberr.ErrIsNil(ctx, err, "更新订单明细商品状态为待发货失败")
				// 支付成功，处理自动发货的商品
				for _, orderGoodsDetail := range orderInfo.OrderGoodsDetail {
					go s.AutoDeliver(gctx.New(), orderGoodsDetail.Id)
				}
			}
			_, err = dao.StoreOrder.Ctx(ctx).Where(dao.StoreOrder.Columns().OrderId, orderInfo.OrderId).Update(orderDo)
			//	记录订单日志
			_, err = dao.StoreOrderLog.Ctx(ctx).Insert(&do.StoreOrderLog{
				OrderId: orderInfo.OrderId,
				Content: data.Message,
				LogType: storeOrderConsts.STORE_ORDER_LOG_TYPE_PAYMENT,
			})
			liberr.ErrIsNil(ctx, err, "日志记录失败")
			// 记录财务流水
			// 使用时间戳作为流水号的一部分
			timestamp := time.Now().UnixNano() / 1e6
			// 生成随机数作为流水号的另一部分
			random := rand.Intn(999999) // 可以根据需要生成不同位数的随机数
			//增加财务流水记录
			err = financeService.StoreFinanceFlow().Add(ctx, &storeFinanceModel.StoreFinanceFlowAddReq{
				Uid:      orderInfo.Uid,
				FlowNo:   gconv.String(timestamp) + gconv.String(random),
				FlowType: gconv.String(storeConsts.FINANCE_TYPE_ORDER),
				Amount:   data.Amount,
				OrderId:  data.OutTradeNo,
				PayType:  data.PayType,
				Remark:   fmt.Sprintf("订单：%d 支付", orderInfo.OrderId),
			})
			liberr.ErrIsNil(ctx, err, "订单流水记录失败")
		})
	})
}

// StatByTheDay 按天统计订单数据
func (s *sStoreOrder) OrderStat(ctx context.Context, req *model.StoreOrderStatReq) (res []*model.StoreOrderStatRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		if req.StartDate == "" {
			req.StartDate = gtime.Now().AddDate(0, 0, -30).Format("Y-m-d 00:00:00")
		}
		if req.EndDate == "" {
			req.EndDate = gtime.Now().Format("Y-m-d 23:59:59")
		}
		// 只统计已支付订单
		err = dao.StoreOrder.Ctx(ctx).
			Fields("YEAR(pay_time) AS year, MONTH(pay_time) AS month, DAY(pay_time) AS day, COUNT(order_id) AS num").
			Where(g.Map{
				"paid":          storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS,
				"pay_time >= ?": req.StartDate,
				"pay_time <= ?": req.EndDate,
			}).Group("YEAR(pay_time), MONTH(pay_time), DAY(pay_time)").Order("YEAR(pay_time) desc, MONTH(pay_time) desc, DAY(pay_time) desc").Scan(&res)
	})
	return
}

// OrderCount 统计订单数量
func (s *sStoreOrder) OrderCount(ctx context.Context, req *model.StoreOrderStatReq) (res int, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		if req.StartDate == "" {
			req.StartDate = gtime.Now().AddDate(0, 0, -30).Format("Y-m-d 00:00:00")
		}
		if req.EndDate == "" {
			req.EndDate = gtime.Now().Format("Y-m-d 23:59:59")
		}
		// 只统计已支付订单
		res, err = dao.StoreOrder.Ctx(ctx).Where(g.Map{
			"paid":          storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS,
			"pay_time >= ?": req.StartDate,
			"pay_time <= ?": req.EndDate,
		}).Count()
	})
	return
}

// SalesAmount 统计订单金额
func (s *sStoreOrder) SalesAmount(ctx context.Context, req *model.StoreOrderStatReq) (res float64, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		if req.StartDate == "" {
			req.StartDate = gtime.Now().AddDate(0, 0, -30).Format("Y-m-d 00:00:00")
		}
		if req.EndDate == "" {
			req.EndDate = gtime.Now().Format("Y-m-d 23:59:59")
		}
		// 只统计已支付订单
		res, err = dao.StoreOrder.Ctx(ctx).Where(g.Map{
			"paid":          storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS,
			"pay_time >= ?": req.StartDate,
			"pay_time <= ?": req.EndDate,
		}).Sum("pay_amount")
	})
	return
}

// AutoCompleteOrder 自动完成订单
func (s *sStoreOrder) AutoCompleteOrder(ctx context.Context) {
	err := g.Try(ctx, func(ctx context.Context) {
		// 检查已发货没有完成的订单
		m := dao.StoreOrder.Ctx(ctx).As("a").
			LeftJoin("store_order_detail b", "b.order_id = a.order_id").
			Where(g.Map{
				"a.status":                    storeOrderConsts.STORE_ORDER_STATUS_DELIVERED,                       // 已发货
				"b.after_sales_status <> ?":   storeOrderConsts.STORE_ORDER_DETAIL_AFTER_SALES_STATUS_APPLY_REFUND, // 未申请退款
				"b.auto_confirm_receipt_at <": gtime.Now(),                                                         // 自动收货时间已过
			})
		m = m.Group("a.order_id")
		count, err := m.Count()
		liberr.ErrIsNil(ctx, err)
		if count == 0 {
			return
		}

		// todo 这里需要自行考虑是否分批次处理,防止一次性处理过多,导致数据库压力过大
		orderIds, err := m.Array("a.order_id")
		liberr.ErrIsNil(ctx, err)
		for _, orderId := range orderIds {
			err = service.StoreOrderDetail().CheckStatus(ctx, gconv.Uint64(orderId))
			liberr.ErrIsNil(ctx, err)
		}
	})
	if err != nil {
		g.Log().Error(ctx, err)
	}
}

// ListMemberOrders 获取用户订单列表
func (s *sStoreOrder) ListMemberOrders(ctx context.Context, req *model.StoreOrderMemberSearchReq) (res *model.StoreOrderMemberSearchRes, err error) {
	res = new(model.StoreOrderMemberSearchRes)
	// todo 实现前端订单查询优化查询条件
	err = g.Try(ctx, func(ctx context.Context) {
		m := dao.StoreOrder.Ctx(ctx).As("a").
			LeftJoin("store_order_detail b", "b.order_id = a.order_id").
			Where("a.uid = ?", req.Uid)
		if req.Status != "" {
			m = m.Where("a."+dao.StoreOrder.Columns().Status+" = ?", gconv.Int(req.Status))
		}
		if req.GoodsName != "" {
			m = m.Where("b.goods_name like ?", "%"+req.GoodsName+"%")
		}
		if req.PayStatus != "" {
			m = m.Where("b.pay_status = ?", req.PayStatus)
		}
		if req.WaitComment {
			// 待评价订单
			m = m.Where("b.commented=0")
		}
		if req.AfterSale {
			// 售后订单
			m = m.Where("b.after_sales_status", storeOrderConsts.STORE_ORDER_DETAIL_AFTER_SALES_STATUS_APPLY_REFUND)
		}
		if len(req.DateRange) > 0 {
			m = m.WhereBetween("a.created_at", req.DateRange[0]+" 00:00:00", req.DateRange[1]+" 23:59:59")
		}
		m = m.Group("a.order_id")
		res.Total, err = m.CountColumn("a.order_id")
		liberr.ErrIsNil(ctx, err, "获取总行数失败")
		if req.PageNum == 0 {
			req.PageNum = 1
		}
		res.CurrentPage = req.PageNum
		if req.PageSize == 0 {
			req.PageSize = consts.PageSize
		}
		sort := "a.created_at desc"
		if req.OrderBy != "" {
			sort = req.OrderBy
		}
		err = m.Fields("a.*").Page(req.PageNum, req.PageSize).WithAll().Order(sort).Scan(&res.List)
		liberr.ErrIsNil(ctx, err, "获取订单列表失败")
	})
	return
}

// GetMemberOrderById 获取用户订单详情
func (s *sStoreOrder) GetMemberOrderById(ctx context.Context, uid uint64, orderId uint64) (res *model.StoreOrderMemberOrderInfoRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		err = dao.StoreOrder.Ctx(ctx).WithAll().Where(dao.StoreOrder.Columns().Uid, uid).Where(dao.StoreOrder.Columns().OrderId, orderId).Scan(&res)
		res.DiscountAmount = gconv.Float64(fmt.Sprintf("%.2f", res.TotalAmount-res.PayAmount))
		liberr.ErrIsNil(ctx, err, "获取订单信息失败")
	})
	return
}

func (s *sStoreOrder) AutoCancelOrder(ctx context.Context, orderId ...uint64) {
	// todo 这里需要实现自动取消订单逻辑
	err := g.Try(ctx, func(ctx context.Context) {
		// 订单超时未支付，自动取消订单
		// 取消条件：订单状态为待支付，订单创建时间超过取消时间
		orderCancelTime := storeCommonService.StoreConfig().GetConfigValueByName(ctx, "cancel_order_time")
		if orderCancelTime == "" {
			orderCancelTime = "10"
		}
		m := dao.StoreOrder.Ctx(ctx).Where(g.Map{
			dao.StoreOrder.Columns().Status:            storeOrderConsts.STORE_ORDER_STATUS_UNPAID,
			dao.StoreOrder.Columns().CreatedAt + " <=": gtime.Now().Add(-gconv.Duration(orderCancelTime) * time.Minute),
		})
		// 如果传了订单号，只取消指定的订单
		if len(orderId) > 0 {
			m = m.Where(dao.StoreOrder.Columns().OrderId+" in (?)", orderId)
		}
		_, err := m.Update(&do.StoreOrder{
			Status: storeOrderConsts.STORE_ORDER_STATUS_CLOSED,
		})
		liberr.ErrIsNil(ctx, err)
	})
	if err != nil {
		g.Log().Error(ctx, err)
	}
}

// AutoDeliver 自动发货
func (s *sStoreOrder) AutoDeliver(ctx context.Context, orderDetailId uint64) {
	// todo 这里需要完善自动发货逻辑
	var err error
	orderGoodsDetail, err := service.StoreOrderDetail().GetById(ctx, orderDetailId)
	if err != nil || orderGoodsDetail == nil {
		err = errors.New("订单详情不存在")
	}
	err = g.DB().Transaction(ctx, func(ctx context.Context, tx gdb.TX) error {
		return g.Try(ctx, func(ctx context.Context) {
			// 获取自动确认收货时间
			autoConfirmDay := gconv.Int(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "auto_confirm_day"))
			// 发货数量
			var deliveryNum = orderGoodsDetail.Quantity - orderGoodsDetail.Delivered
			if deliveryNum > 0 {
				// 商品属性信息
				goodsAttrInfo, err := goodsService.StoreGoodsAttrValue().GetById(ctx, orderGoodsDetail.GoodsAttr)
				liberr.ErrIsNil(ctx, err, "获取商品属性信息失败")
				switch goodsAttrInfo.Goods.GoodsType {
				case storeGoods.STORE_GOODS_CARDPASSWORD_GOODS:
					// 卡密商品发货
					err = goodsService.StoreGoodsVirtual().Deliver(ctx, orderGoodsDetail.GoodsAttr, deliveryNum, orderGoodsDetail.OrderId, orderGoodsDetail.Uid)
					liberr.ErrIsNil(ctx, err)
				case storeGoods.STORE_GOODS_COUPON_GOODS:
					// 优惠券发货
					err = storeCommonService.StoreCoupon().SendCouponMember(ctx, &storeCommonModel.SendCouponMemeberReq{
						MemberIds: []uint64{orderGoodsDetail.Uid},
						CouponId:  goodsAttrInfo.CouponId,
					}, "自动发货")
					liberr.ErrIsNil(ctx, err)
				default:
					// 非自动发货商品直接返回
					return
				}

				storeDeliverData := &do.StoreDelivery{
					OrderDetailId: orderGoodsDetail.Id,
					OrderId:       orderGoodsDetail.OrderId,
					DeliveryType:  storeCommonConsts.STORE_DELIVERY_TYPE_AUTO_DELIVERY,
					Remark:        "自动发货",
					DeliveryNum:   deliveryNum,
				}
				storeDeliverData.Id, err = commonService.SnowID().GenID()
				liberr.ErrIsNil(ctx, err, "生成ID失败")
				_, err = dao.StoreDelivery.Ctx(ctx).Insert(storeDeliverData)
				liberr.ErrIsNil(ctx, err)
				// 更新订单明细状态
				_, err = dao.StoreOrderDetail.Ctx(ctx).WherePri(orderGoodsDetail.Id).Update(&do.StoreOrderDetail{
					Delivered:            orderGoodsDetail.Quantity,
					Status:               storeOrderConsts.STORE_ORDER_DETAIL_STATUS_WAITRECEIVING,
					AutoConfirmReceiptAt: gtime.Now().AddDate(0, 0, autoConfirmDay),
				})
				liberr.ErrIsNil(ctx, err)
				//更新订单状态
				_, err = dao.StoreOrder.Ctx(ctx).Where(dao.StoreOrder.Columns().OrderId, orderGoodsDetail.OrderId).Update(&do.StoreOrder{
					Status: storeOrderConsts.STORE_ORDER_DETAIL_STATUS_WAITRECEIVING,
				})
				liberr.ErrIsNil(ctx, err)

				//常量判断送货方式
				var deliveryType string
				switch storeDeliverData.DeliveryType {
				case storeCommonConsts.STORE_DELIVERY_TYPE_EXPRESS:
					deliveryType = "快递配送"
					break
				case storeCommonConsts.STORE_DELIVERY_TYPE_HOME_DELIVERY:
					deliveryType = "送货上门"
					break
				case storeCommonConsts.STORE_DELIVERY_TYPE_SELF_PICKUP:
					deliveryType = "自提"
					break
				case storeCommonConsts.STORE_DELIVERY_TYPE_NO_DELIVERY:
					deliveryType = "无需配送"
					break
				case storeCommonConsts.STORE_DELIVERY_TYPE_AUTO_DELIVERY:
					deliveryType = "自动发货"
					break
				default:
					deliveryType = "无法确定送货方式"
				}
				// 发送通知给用户
				err = storeMemberService.StoreMemberNotification().Add(ctx, &storeMemberModel.StoreMemberNotificationAddReq{
					Uid:   orderGoodsDetail.Uid,
					Title: "商品已发货",
					//Content: `{"orderId":"` + gconv.String(orderGoodsDetail.OrderId) + `","goodsName":"` + orderGoodsDetail.GoodsName + `","goodsNum":"` + gconv.String(deliveryNum) + `"}`,
					Content: `订单编号：` + gconv.String(orderGoodsDetail.OrderId) + `<br/>送货方式：` + deliveryType + `<br/>商品名称：` + orderGoodsDetail.GoodsName + `<br/>数量：` + gconv.String(deliveryNum),
					Type:    storeMemberConsts.STORE_MEMBER_NOTIFICATION_TYPE_LOGISTICS,
					Author:  "系统通知",
				})
				liberr.ErrIsNil(ctx, err)
			}
		})
	})
	if err != nil {
		fmt.Printf("%+v", err)
		// 自动发货失败，发出事件和通知
		err = systemService.SysNotice().Add(ctx, &systemModel.SysNoticeAddReq{
			Title:   "自动发货失败",
			Type:    systemConsts.SysNoticeType,
			Content: fmt.Sprintf("商品自动发货失败，订单号：%d，商品名称：%s，商品数量：%d，错误信息：%s", orderGoodsDetail.OrderId, orderGoodsDetail.GoodsName+orderGoodsDetail.GoodsAttrName, orderGoodsDetail.Quantity, err.Error()),
		})
		commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_ORDER_DETAIL_AUTO_DELIVER_FAIL, ctx, orderDetailId)
	}
}

// Confirm 确认订单
func (s *sStoreOrder) Confirm(ctx context.Context, memberInfo *storeMemberModel.StoreMemberInfoRes, req *model.StoreOrderConfirmReq) (resData *model.StoreOrderConfirmRes, err error) {
	// 临时记录订单总金额，计算优惠券时需要使用总额来计算
	resData = new(model.StoreOrderConfirmRes)
	var totalAmount = 0.0
	err = g.Try(ctx, func(ctx context.Context) {
		// 保存商品列表信息，用于计算优惠券时使用
		var orderDetailAddReqList []*model.StoreOrderDetailAddReq
		for _, orderGoods := range req.GoodsList {
			goodsAttrValue, err := goodsService.StoreGoodsAttrValue().GetById(ctx, orderGoods.GoodsAttr)
			liberr.ErrIsNil(ctx, err, "获取商品属性信息失败")
			if goodsAttrValue == nil || goodsAttrValue.Goods == nil {
				liberr.ErrIsNil(ctx, errors.New("商品属性信息无效"))
			}
			if goodsAttrValue.Goods.IsShow != 1 {
				liberr.ErrIsNil(ctx, errors.New(fmt.Sprintf("商品：%s 已下架", goodsAttrValue.Goods.Name)))
			}
			tmpOrderDetailAddReq := &model.StoreOrderDetailAddReq{
				Uid:           memberInfo.Uid,                                                                                          // 用户ID
				GoodsId:       goodsAttrValue.GoodsId,                                                                                  // 商品ID
				GoodsAttr:     goodsAttrValue.Id,                                                                                       // 商品属性ID
				GoodsName:     goodsAttrValue.Goods.Name,                                                                               // 商品名称
				GoodsImg:      goodsAttrValue.Goods.Thumb,                                                                              // 商品属性缩略图
				GoodsUnit:     goodsAttrValue.Goods.UnitName,                                                                           // 商品单位
				GoodsAttrName: goodsAttrValue.Suk,                                                                                      // 商品属性名称
				GoodsPrice:    goodsAttrValue.Price,                                                                                    // 商品价格
				CartId:        orderGoods.CartId,                                                                                       // 购物车ID
				Quantity:      orderGoods.Quantity,                                                                                     // 购买数量
				RealPrice:     goodsService.StoreGoodsAttrValue().CalculatePrice(ctx, memberInfo, goodsAttrValue, orderGoods.Quantity), // 商品实际价格
				GoodsCategory: goodsAttrValue.Goods.CateId,                                                                             // 商品分类ID
			}

			// 修改订单总额 = 商品实际价格 * 数量
			totalAmount += gconv.Float64(tmpOrderDetailAddReq.RealPrice) * gconv.Float64(orderGoods.Quantity)
			orderDetailAddReqList = append(orderDetailAddReqList, tmpOrderDetailAddReq)
		}
		// 先保存没有优惠过的订单总额
		resData.TotalAmount = totalAmount
		// todo 扣减信息计算包含积分，优惠券，满减，满折，会员折扣，首单等============================================

		// 使用优惠券
		if len(req.Coupons) > 0 {
			couponSet := gset.NewStrSetFrom(req.Coupons, true)
			if couponSet.Size() > 0 {
				for _, couponNumber := range couponSet.Slice() {
					var couponInfo *storeMemberModel.StoreMemberCouponInfoRes
					couponInfo, err = storeMemberService.StoreMemberCoupon().GetByCouponNumber(ctx, couponNumber, memberInfo.Uid)
					if err != nil || couponInfo == nil {
						liberr.ErrIsNil(ctx, errors.New("优惠券不存在或无效"))
					}
					// 是否允许叠加使用
					if couponSet.Size() > 1 && couponInfo.Coupon.AllowStack == 0 {
						liberr.ErrIsNil(ctx, errors.New(fmt.Sprintf("优惠券%s不允许叠加使用", couponInfo.CouponNumber)))
					}
					err = storeMemberService.StoreMemberCoupon().IsValid(ctx, couponInfo, resData.TotalAmount)
					liberr.ErrIsNil(ctx, err)
					// 计算优惠券抵扣，这里使用的是订单的总金额来进行计算,不能使用totalAmount，因为这个可能是已经被优惠过的
					couponDiscountAmount, err := service.StoreOrderDiscount().CalculateCouponDiscount(ctx, couponInfo, resData.TotalAmount, orderDetailAddReqList)
					liberr.ErrIsNil(ctx, err)
					if couponDiscountAmount > 0 {
						totalAmount -= couponDiscountAmount
						// 如果优惠券抵扣后金额小于0，则总金额为0
						if totalAmount < 0 {
							totalAmount = 0
						}
						resData.CouponAmount += couponDiscountAmount
					}
				}
			}
		}

		if req.Points > 0 {
			if gconv.Int(req.Points) > memberInfo.Points {
				liberr.ErrIsNil(ctx, errors.New("积分不足"))
			}
			pointsDiscountAmount, usedPoints, err := storeMemberService.StoreMemberPoints().CalculatePointsDiscount(ctx, req.Points, totalAmount)
			liberr.ErrIsNil(ctx, err)
			totalAmount -= pointsDiscountAmount
			resData.PointsAmount = pointsDiscountAmount
			resData.UsedPoints = usedPoints
		}
		// 计算扣减信息结束=========================================================================================

		// 计算运费
		resData.FreightAmount, err = freightService.StoreFreightTemplate().Freight(ctx, req.AddressId, req.GoodsList)
		liberr.ErrIsNil(ctx, err, "计算运费失败")

		resData.PayFreight = resData.FreightAmount
		// 应付订单总额
		resData.PayAmount, err = strconv.ParseFloat(strconv.FormatFloat(totalAmount, 'f', 2, 64), 64)
		liberr.ErrIsNil(ctx, err, "计算应付订单总额失败")
	})
	return
}

// 获取用户某种状态下的订单数
func (s *sStoreOrder) MemberOrdersStatusCount(ctx context.Context, uid uint64, status int) (count int, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		count, err = dao.StoreOrder.Ctx(ctx).WithAll().Where(dao.StoreOrder.Columns().Uid+"=?", uid).Where(dao.StoreOrder.Columns().Status+"=?", status).Count()
		liberr.ErrIsNil(ctx, err, "获取总数失败")
	})
	return
}
