/**
 * @Company: 云南奇讯科技有限公司
 * @Author: yxf
 * @Description: 订单列表
 * @Date: 2024-01-16 12:00:38
 */
package logic

import (
	"context"
	"errors"
	"fmt"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/util/gconv"
	commonService "github.com/tiger1103/gfast/v3/internal/app/common/service"
	storeEventConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeEvent"
	storeOrderConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeOrder"
	model2 "github.com/tiger1103/gfast/v3/internal/app/store/finance/model"
	memberDao "github.com/tiger1103/gfast/v3/internal/app/store/member/dao"
	storeMemberModel "github.com/tiger1103/gfast/v3/internal/app/store/member/model"
	orderDao "github.com/tiger1103/gfast/v3/internal/app/store/order/dao"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/model"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/service"
	storeCommonModel "github.com/tiger1103/gfast/v3/internal/app/store/storeCommon/model"
	storeCommonService "github.com/tiger1103/gfast/v3/internal/app/store/storeCommon/service"
	wechatModel "github.com/tiger1103/gfast/v3/internal/app/store/wechat/model"
	wechatPayService "github.com/tiger1103/gfast/v3/internal/app/store/wechat/service"
	"github.com/tiger1103/gfast/v3/library/liberr"
)

func init() {
	service.RegisterStorePayment(New())
}

func New() service.IStorePayment {
	return &sStorePayment{}
}

type sStorePayment struct{}

func (s *sStorePayment) WechatNativePay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (codeUrl string, err error) {
	//TODO 需要实现微信扫码支付功能
	err = g.Try(ctx, func(ctx context.Context) {
		if storeCommonService.StoreConfig().GetConfigValueByName(ctx, "wechat_pay_native") != "1" {
			liberr.ErrIsNil(ctx, errors.New("不支持微信扫码支付"))
		}
		// 读取微信支付配置
		var wxpayConfig *storeCommonModel.StoreConfigPaymentWxpay
		err = gconv.Struct(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "wechat_pay_config"), &wxpayConfig)
		liberr.ErrIsNil(ctx, err, "获取微信支付配置失败")

		// 调用微信支付接口
		description := orderInfo.OrderGoodsDetail[0].GoodsName
		if len(orderInfo.OrderGoodsDetail) > 1 {
			description += "等多个商品"
		}
		codeUrl, err = wechatPayService.WechatPay().Native(ctx, &wechatModel.WechatPayPayParams{
			Amount:      gconv.Int(orderInfo.PayAmount * 100),
			OutTradeNo:  gconv.String(orderInfo.OutTradeNo),
			Attach:      "order",
			Description: description,
		})
		liberr.ErrIsNil(ctx, err)
	})
	return
}

func (s *sStorePayment) WechatFriendPay(ctx context.Context, openid string, orderInfo *model.StoreOrderInfoRes) (jssdkConfig interface{}, err error) {
	//TODO 需要实现微信好友支付功能
	return nil, errors.New("暂不支持微信好友支付")
}

func (s *sStorePayment) WechatPay(ctx context.Context, openid string, wechatPayType int, orderInfo *model.StoreOrderInfoRes) (jssdkConfig interface{}, err error) {
	// todo 需要实现微信支付功能
	err = g.Try(ctx, func(ctx context.Context) {
		if openid == "" {
			liberr.ErrIsNil(ctx, errors.New("openid不能为空"))
		}
		if storeCommonService.StoreConfig().GetConfigValueByName(ctx, "wechat_pay") != "1" {
			liberr.ErrIsNil(ctx, errors.New("不支持微信支付"))
		}
		// 读取微信支付配置
		var wxpayConfig *storeCommonModel.StoreConfigPaymentWxpay
		err = gconv.Struct(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "wechat_pay_config"), &wxpayConfig)
		liberr.ErrIsNil(ctx, err, "获取微信支付配置失败")

		// 调用微信支付接口
		description := orderInfo.OrderGoodsDetail[0].GoodsName
		if len(orderInfo.OrderGoodsDetail) > 1 {
			description += "等多个商品"
		}
		if wechatPayType == 0 { //  jssdk 支付
			jssdkConfig, err = wechatPayService.WechatPay().Prepay(ctx, &wechatModel.WechatPayPayParams{
				Amount:      gconv.Int(orderInfo.PayAmount * 100),
				OutTradeNo:  gconv.String(orderInfo.OutTradeNo),
				OpenId:      openid,
				Attach:      "order",
				Description: description,
			})
			liberr.ErrIsNil(ctx, err)
		}
		if wechatPayType == 1 { //  小程序支付
			jssdkConfig, err = wechatPayService.WechatPay().AppletMiniPay(ctx, &wechatModel.WechatPayPayParams{
				Amount:      gconv.Int(orderInfo.PayAmount * 100),
				OutTradeNo:  gconv.String(orderInfo.OutTradeNo),
				OpenId:      openid,
				Attach:      "order",
				Description: description,
			})
			liberr.ErrIsNil(ctx, err)
		}

	})
	return
}

// 微信余额充值
func (s *sStorePayment) WeChatRecharge(ctx context.Context, openid string, wechatPayType int, rechargeInfo *model2.StoreFinanceRechargeLogInfoRes) (jssdkConfig interface{}, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		if openid == "" {
			liberr.ErrIsNil(ctx, errors.New("openid不能为空"))
		}
		if storeCommonService.StoreConfig().GetConfigValueByName(ctx, "wechat_pay") != "1" {
			liberr.ErrIsNil(ctx, errors.New("不支持微信支付"))
		}
		// 读取微信支付配置
		var wxpayConfig *storeCommonModel.StoreConfigPaymentWxpay
		err = gconv.Struct(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "wechat_pay_config"), &wxpayConfig)

		liberr.ErrIsNil(ctx, err, "获取微信支付配置失败")

		// 调用微信支付接口
		description := "用户余额充值"

		if wechatPayType == 0 { //jssdk支付
			jssdkConfig, err = wechatPayService.WechatPay().Prepay(ctx, &wechatModel.WechatPayPayParams{
				Amount:      gconv.Int(rechargeInfo.RechargeAmount * 100),
				OutTradeNo:  gconv.String(rechargeInfo.PayNo),
				OpenId:      openid,
				Attach:      "balance",
				Description: description,
			})
			liberr.ErrIsNil(ctx, err)
		}

		if wechatPayType == 1 { //小程序支付
			jssdkConfig, err = wechatPayService.WechatPay().AppletMiniPay(ctx, &wechatModel.WechatPayPayParams{
				Amount:      gconv.Int(rechargeInfo.RechargeAmount * 100),
				OutTradeNo:  gconv.String(rechargeInfo.PayNo),
				OpenId:      openid,
				Attach:      "balance",
				Description: description,
			})
			liberr.ErrIsNil(ctx, err)
		}

	})
	return
}

func (s *sStorePayment) Alipay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (err error) {
	// todo 需要实现支付宝支付功能，需要配置支付回调功能
	return g.Try(ctx, func(ctx context.Context) {
		// 读取支付宝支付配置
		alipayPay := storeCommonService.StoreConfig().GetConfigValueByName(ctx, "alipay_pay")
		if alipayPay != "1" {
			liberr.ErrIsNil(ctx, errors.New("不支持支付宝"))
		}
		var alipayConfig *storeCommonModel.StoreConfigPaymentAlipay
		err = gconv.Struct(storeCommonService.StoreConfig().GetConfigValueByName(ctx, "alipay_config"), &alipayConfig)
		liberr.ErrIsNil(ctx, err, "获取支付宝配置失败")

	})
}

// BalancePay 余额支付
func (s *sStorePayment) BalancePay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (err error) {
	// 余额支付，需要检查用户余额是否足够
	err = g.Try(ctx, func(ctx context.Context) {
		var memberInfo *storeMemberModel.StoreMemberInfoRes
		err = memberDao.StoreMember.Ctx(ctx).Where("uid", orderInfo.Uid).Scan(&memberInfo)
		if memberInfo == nil {
			liberr.ErrIsNil(ctx, errors.New("没有获取到用户信息"))
		}
		liberr.ErrIsNil(ctx, err, "获取用户信息失败")
		totalAmount := orderInfo.PayAmount + orderInfo.PayFreight
		if totalAmount > 0 {
			if memberInfo.Balance < totalAmount {
				liberr.ErrIsNil(ctx, errors.New("余额不足"))
			}
		}
		// 扣除用户余额
		_, err = memberDao.StoreMember.Ctx(ctx).
			Where("uid", orderInfo.Uid).
			Where("`balance` >= `balance` - "+fmt.Sprintf("%.2f", totalAmount)).
			Decrement(memberDao.StoreMember.Columns().Balance, totalAmount)
		// 更新订单支付状态
		eventData := &storeCommonModel.StorePaymentEventData{
			OutTradeNo: gconv.String(orderInfo.OutTradeNo),
			PayType:    orderInfo.PayType,
			Message:    "订单支付成功",
			Paid:       storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS,
			Amount:     totalAmount,
			Attach:     "order",
		}
		if err != nil {
			eventData.Paid = storeOrderConsts.STORE_ORDER_PAY_STATUS_FAILURE
			eventData.Message = "订单支付失败：" + err.Error()
		}

		commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_PAYMENT_STATUS_CHANGE, ctx, eventData)
		liberr.ErrIsNil(ctx, err, "扣除用户余额失败")
	})
	return
}

func (s *sStorePayment) OfflinePay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (err error) {
	// 选择线下支付，只需要更新订单支付类型
	_, err = orderDao.StoreOrder.Ctx(ctx).Where("order_id", orderInfo.OrderId).Update(g.Map{"pay_type": orderInfo.PayType})
	return
}
