/**
 * @Company: 云南奇讯科技有限公司
 * @Author: yxf
 * @Description: 微信支付
 * @Date: 2024/8/12 上午11:47
 */

package wechat_pay

import (
	"context"
	"errors"
	"fmt"
	"github.com/ArtisanCloud/PowerWeChat/v3/src/kernel/models"
	"github.com/ArtisanCloud/PowerWeChat/v3/src/payment"
	"github.com/ArtisanCloud/PowerWeChat/v3/src/payment/notify/request"
	powerWechatOrderRequest "github.com/ArtisanCloud/PowerWeChat/v3/src/payment/order/request"
	responseOrder "github.com/ArtisanCloud/PowerWeChat/v3/src/payment/order/response"
	powerWechatPartnerRequest "github.com/ArtisanCloud/PowerWeChat/v3/src/payment/partner/request"
	"github.com/ArtisanCloud/PowerWeChat/v3/src/payment/reverse/response"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/os/gctx"
	"github.com/gogf/gf/v2/util/gconv"
	"github.com/tiger1103/gfast/v3/api/v1/store/wechat"
	commonService "github.com/tiger1103/gfast/v3/internal/app/common/service"
	storeEventConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeEvent"
	storeOrderConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeOrder"
	wechatConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/wechat"
	storeCommonModel "github.com/tiger1103/gfast/v3/internal/app/store/storeCommon/model"
	storeCommonService "github.com/tiger1103/gfast/v3/internal/app/store/storeCommon/service"
	"github.com/tiger1103/gfast/v3/internal/app/store/wechat/model"
	"github.com/tiger1103/gfast/v3/internal/app/store/wechat/service"
	"github.com/tiger1103/gfast/v3/library/liberr"
	"net/http"
)

// 初始化微信支付相关 service
func init() {

	// 先初始化一次，防止没有配置时无法启动
	service.RegisterWechatPay(NewWechatPay())
}

var WechatPay = new(wechatPay)

type wechatPay struct {
	PaymentService   *payment.Payment
	WechatPayConfig  *model.WechatPayConfig
	AppletPayService *payment.Payment
}

func NewWechatPay() service.IWechatPay {
	ctx := gctx.New()
	//读取微信支付配置
	wechatConfig, wechatConfigErr := storeCommonService.StoreConfig().GetByName(context.Background(), "wechat_pay_config")

	if wechatConfigErr != nil {
		g.Log().Error(context.Background(), wechatConfigErr)
	}
	var wechatPayConfig *model.WechatPayConfig
	wechatConfigErr = gconv.Struct(wechatConfig.ConfigValue, &wechatPayConfig)
	if wechatConfigErr != nil {
		g.Log().Error(context.Background(), errors.New("wechat_pay_config 获取配置错误"))
	}

	userConfig := &payment.UserConfig{

		AppID:       wechatPayConfig.AppId,
		MchID:       wechatPayConfig.Mchid,
		MchApiV3Key: wechatPayConfig.ApiV3Key,
		CertPath:    "./" + g.Cfg().MustGet(ctx, "server.serverRoot").String() + "/" + wechatPayConfig.Cert,
		KeyPath:     "./" + g.Cfg().MustGet(ctx, "server.serverRoot").String() + "/" + wechatPayConfig.CertKey,
		SerialNo:    wechatPayConfig.CertSerial,
		Log: payment.Log{
			Level: g.Cfg("store").MustGet(ctx, "wechatPay.log.level", "").String(),
			File:  g.Cfg("store").MustGet(ctx, "wechatPay.log.file", "").String(),
		},
		HttpDebug: g.Cfg("store").MustGet(ctx, "wechatPay.httpDebug", false).Bool(),
		NotifyURL: g.Cfg("store").MustGet(ctx, "wechatPay.notifyUrl").String(),
		Sandbox:   g.Cfg("store").MustGet(ctx, "wechatPay.sandbox", false).Bool(),
	}
	// 微信支付类型为服务商模式时，设置子商户号
	if wechatPayConfig.WxpayType == wechatConsts.WECHAT_PAY_TYPE_PARTNER {
		userConfig.SubMchID = wechatPayConfig.SubMchid
	}
	// 微信支付类型为小程序非商户号绑定模式时，设置子商户号
	if wechatPayConfig.MinProgramMchidType == wechatConsts.WECHAT_PAY_MINPROGRAM_TYPE_NOMERCHANT {
		userConfig.SubMchID = wechatPayConfig.MinProgramMchid
	}
	newPayment, err := payment.NewPayment(userConfig)

	if err != nil {
		g.Log().Error(ctx, err)
	}
	appletConfig := userConfig
	appletConfig.AppID = wechatPayConfig.WxAppid
	appletPayment, err := payment.NewPayment(appletConfig)

	return &wechatPay{
		PaymentService:   newPayment,
		WechatPayConfig:  wechatPayConfig,
		AppletPayService: appletPayment,
	}
}

// AppletMiniPay 微信支付接口  小程序支付
func (s *wechatPay) AppletMiniPay(ctx context.Context, params *model.WechatPayPayParams) (res *model.WechatPayPrepayConfig, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		// 创建预支付订单
		var prepayID string

		if s.WechatPayConfig.WxpayType == wechatConsts.WECHAT_PAY_TYPE_PARTNER {
			// 订单支付子商户号模式
			prepayRes, payErr := s.AppletPayService.Partner.JSAPITransaction(ctx, &powerWechatPartnerRequest.RequestJSAPIPrepay{
				Amount:      &powerWechatPartnerRequest.JSAPIAmount{Total: params.Amount, Currency: "CNY"},
				Description: params.Description,
				OutTradeNo:  params.OutTradeNo,
				Attach:      params.Attach,
				Payer:       &powerWechatPartnerRequest.JSAPIPayer{SpOpenId: params.OpenId},
			})
			liberr.ErrIsNil(ctx, payErr, "创建预支付订单失败")
			if prepayRes.PrepayID == "" {
				liberr.ErrIsNil(ctx, errors.New(prepayRes.Message))
			} else {
				prepayID = prepayRes.PrepayID
			}

		} else {
			// 订单支付商户号模式
			prepayRes, payErr := s.AppletPayService.Order.JSAPITransaction(ctx, &powerWechatOrderRequest.RequestJSAPIPrepay{
				Amount:      &powerWechatOrderRequest.JSAPIAmount{Total: params.Amount, Currency: "CNY"},
				Description: params.Description,
				OutTradeNo:  params.OutTradeNo,
				Attach:      params.Attach,
				Payer:       &powerWechatOrderRequest.JSAPIPayer{OpenID: params.OpenId},
			})
			liberr.ErrIsNil(ctx, payErr, "创建预支付订单失败")
			if prepayRes.PrepayID == "" {
				liberr.ErrIsNil(ctx, errors.New(prepayRes.Message))
			} else {
				prepayID = prepayRes.PrepayID
			}
		}
		// 根据prepay_id, 拼接出jssdk所需要的config
		data, payErr := s.AppletPayService.JSSDK.BridgeConfig(prepayID, true)
		liberr.ErrIsNil(ctx, payErr, "获取jssdk配置失败")
		payErr = gconv.Struct(data, &res)
		liberr.ErrIsNil(ctx, payErr, "解析WechatPayPrepayConfig配置失败")
		g.Log().Debug(ctx, "wechat pay prepay config", res)
	})
	return
}

// Prepay 微信支付接口  公众号
func (s *wechatPay) Prepay(ctx context.Context, params *model.WechatPayPayParams) (res *model.WechatPayPrepayConfig, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		// 创建预支付订单
		var prepayID string

		if s.WechatPayConfig.WxpayType == wechatConsts.WECHAT_PAY_TYPE_PARTNER {
			// 订单支付子商户号模式
			prepayRes, payErr := s.PaymentService.Partner.JSAPITransaction(ctx, &powerWechatPartnerRequest.RequestJSAPIPrepay{
				Amount:      &powerWechatPartnerRequest.JSAPIAmount{Total: params.Amount, Currency: "CNY"},
				Description: params.Description,
				OutTradeNo:  params.OutTradeNo,
				Attach:      params.Attach,
				Payer:       &powerWechatPartnerRequest.JSAPIPayer{SpOpenId: params.OpenId},
			})
			liberr.ErrIsNil(ctx, payErr, "创建预支付订单失败")
			if prepayRes.PrepayID == "" {
				liberr.ErrIsNil(ctx, errors.New(prepayRes.Message))
			} else {
				prepayID = prepayRes.PrepayID
			}

		} else {
			// 订单支付商户号模式
			prepayRes, payErr := s.PaymentService.Order.JSAPITransaction(ctx, &powerWechatOrderRequest.RequestJSAPIPrepay{
				Amount:      &powerWechatOrderRequest.JSAPIAmount{Total: params.Amount, Currency: "CNY"},
				Description: params.Description,
				OutTradeNo:  params.OutTradeNo,
				Attach:      params.Attach,
				Payer:       &powerWechatOrderRequest.JSAPIPayer{OpenID: params.OpenId},
			})
			liberr.ErrIsNil(ctx, payErr, "创建预支付订单失败")
			if prepayRes.PrepayID == "" {
				liberr.ErrIsNil(ctx, errors.New(prepayRes.Message))
			} else {
				prepayID = prepayRes.PrepayID
			}
		}
		// 根据prepay_id, 拼接出jssdk所需要的config
		data, payErr := s.PaymentService.JSSDK.BridgeConfig(prepayID, true)
		liberr.ErrIsNil(ctx, payErr, "获取jssdk配置失败")
		payErr = gconv.Struct(data, &res)
		liberr.ErrIsNil(ctx, payErr, "解析WechatPayPrepayConfig配置失败")
		g.Log().Debug(ctx, "wechat pay prepay config", res)
	})
	return
}

// Native 扫码支付接口
func (s *wechatPay) Native(ctx context.Context, params *model.WechatPayPayParams) (codeUrl string, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		// 创建预支付订单
		if s.WechatPayConfig.WxpayType == wechatConsts.WECHAT_PAY_TYPE_PARTNER {
			resp, err := s.PaymentService.Partner.TransactionNative(ctx, &powerWechatPartnerRequest.RequestNativePrepay{
				Description: params.Description,
				OutTradeNo:  params.OutTradeNo,
				Attach:      params.Attach,
				Amount: &powerWechatPartnerRequest.NativeAmount{
					Total:    params.Amount,
					Currency: "CNY",
				},
			})
			liberr.ErrIsNil(ctx, err, "创建预支付订单失败")
			if resp.CodeURL != "" {
				codeUrl = resp.CodeURL
			} else {
				if resp.Message != "" {
					liberr.ErrIsNil(ctx, errors.New(resp.Message))
				}
				if resp.ErrMsg != "" {
					liberr.ErrIsNil(ctx, errors.New(resp.ErrMsg))
				}
			}
		} else {
			resp, err := s.PaymentService.Order.TransactionNative(ctx, &powerWechatOrderRequest.RequestNativePrepay{
				Description: params.Description,
				OutTradeNo:  params.OutTradeNo,
				Attach:      params.Attach,
				Amount: &powerWechatOrderRequest.NativeAmount{
					Total:    params.Amount,
					Currency: "CNY",
				},
			})
			liberr.ErrIsNil(ctx, err, "创建预支付订单失败")
			if resp.CodeURL != "" {
				codeUrl = resp.CodeURL
			} else {
				if resp.Message != "" {
					liberr.ErrIsNil(ctx, errors.New(resp.Message))
				}
				if resp.ErrMsg != "" {
					liberr.ErrIsNil(ctx, errors.New(resp.ErrMsg))
				}
			}
		}
	})
	return
}

// QueryOrder 订单查询接口
func (s *wechatPay) QueryOrder(ctx context.Context, outTradeNo string) (res *wechat.WechatPayQuerOrderRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		var resOrder *responseOrder.ResponseOrder
		resOrder, err = s.PaymentService.Order.QueryByOutTradeNumber(ctx, outTradeNo)
		if err != nil {
			return
		}
		res = &wechat.WechatPayQuerOrderRes{
			AmountTotal:    gconv.Int(resOrder.Amount.Total),
			Openid:         resOrder.Payer,
			TradeState:     resOrder.TradeState,
			TradeType:      resOrder.TradeType,
			TransactionId:  resOrder.TransactionID,
			TradeStateDesc: resOrder.TradeStateDesc,
			SuccessTime:    resOrder.SuccessTime,
		}
	})
	return
}

// Notify 支付结果通知接口
func (s *wechatPay) Notify(ctx context.Context, r *http.Request) (err error) {
	_, err = s.PaymentService.HandlePaidNotify(r, func(message *request.RequestNotify, transaction *models.Transaction, fail func(message string)) interface{} {
		eventData := &storeCommonModel.StorePaymentEventData{
			OutTradeNo:    transaction.OutTradeNo,
			Message:       message.Summary,
			OpenId:        transaction.Payer.OpenID,
			PayType:       storeOrderConsts.STORE_PAYTYPE_WECHAT_PAY,
			TransactionId: transaction.TransactionID,
			Attach:        transaction.Attach,
		}
		if message.EventType != "TRANSACTION.SUCCESS" {
			// 支付不成功
			eventData.Paid = storeOrderConsts.STORE_ORDER_PAY_STATUS_FAILURE
			commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_PAYMENT_STATUS_CHANGE, ctx, eventData)
			g.Log().Debug(ctx, "wechat pay notify success event data", eventData)
			return true
		}
		if transaction.OutTradeNo != "" {
			// 支付成功
			eventData.Amount = gconv.Float64(transaction.Amount.Total / 100)
			eventData.Paid = storeOrderConsts.STORE_ORDER_PAY_STATUS_SUCCESS
			// 发布支付成功事件
			commonService.EventBus().Publish(storeEventConsts.STORE_EVENT_PAYMENT_STATUS_CHANGE, ctx, eventData)
			g.Log().Debug(ctx, fmt.Sprintf("订单号：%s 支付成功", transaction.OutTradeNo))
		} else {
			// 因为微信这个回调不存在订单号，所以可以告诉微信我还没处理成功，等会它会重新发起通知
			// 如果不需要，直接返回true即可
			fail("payment fail")
		}
		return true
	})
	return
}

// Refunds 退款接口
func (s *wechatPay) Refunds(ctx context.Context) (err error) {
	return
}

// QueryRefund 退款查询接口
func (s *wechatPay) QueryRefund(ctx context.Context) (err error) {
	return
}

// Close 关闭订单接口
func (s *wechatPay) Close(ctx context.Context, outTradeNo string) (res *response.ResponseReserve, err error) {
	res, err = s.PaymentService.Reverse.ByOutTradeNumber(ctx, outTradeNo)
	g.Log().Debug(ctx, "wechat pay close response", res)
	return
}
