// ==========================================================================
// desc:用户端订单功能
// company:云南奇讯科技有限公司
// ==========================================================================

package controller

import (
	"context"
	"errors"
	"fmt"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/tiger1103/gfast/v3/api/v1/storeApp/order"
	storeOrderConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/storeOrder"
	storeMemberService "github.com/tiger1103/gfast/v3/internal/app/store/member/service"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/model"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/model/do"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/service"
	systemController "github.com/tiger1103/gfast/v3/internal/app/system/controller"
	ucenterService "github.com/tiger1103/gfast/v3/internal/app/ucenter/service"
	"github.com/tiger1103/gfast/v3/library/liberr"
)

type storeOrderController struct {
	systemController.BaseController
}

var StoreOrder = new(storeOrderController)

// List 列表
func (c *storeOrderController) List(ctx context.Context, req *order.StoreOrderMemberSearchReq) (res *order.StoreOrderMemberSearchRes, err error) {
	req.StoreOrderMemberSearchReq.Uid = ucenterService.Context().GetMemberId(ctx)
	res = new(order.StoreOrderMemberSearchRes)
	res.StoreOrderMemberSearchRes, err = service.StoreOrder().ListMemberOrders(ctx, req.StoreOrderMemberSearchReq)
	return
}

// Get 获取订单
func (c *storeOrderController) Get(ctx context.Context, req *order.StoreOrderMemberGetReq) (res *order.StoreOrderMemberGetRes, err error) {
	res = new(order.StoreOrderMemberGetRes)
	res.StoreOrderMemberOrderInfoRes, err = service.StoreOrder().GetMemberOrderById(ctx, ucenterService.Context().GetMemberId(ctx), req.OrderId)
	if res.StoreOrderMemberOrderInfoRes == nil {
		return nil, errors.New("订单不存在")
	}
	return
}

// Create 创建订单
func (c *storeOrderController) Create(ctx context.Context, req *order.StoreOrderCreateReq) (res *order.StoreOrderCreateRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		memberInfo, err := storeMemberService.StoreMember().GetByUid(ctx, ucenterService.Context().GetMemberId(ctx))
		liberr.ErrIsNil(ctx, err)
		pass, err := service.StoreOrder().FilterCreateBefore(ctx, req, memberInfo)
		liberr.ErrIsNil(ctx, err)
		if !pass {
			liberr.ErrIsNil(ctx, errors.New("请勿重复提交订单"))
		}
		res, err = service.StoreOrder().Create(ctx, memberInfo, req)
		liberr.ErrIsNil(ctx, err)
		err = service.StoreOrderLog().Add(ctx, &do.StoreOrderLog{
			OrderId: res.OrderId,
			Content: "用户创建订单成功",
			LogType: storeOrderConsts.STORE_ORDER_LOG_TYPE_CREATE,
			Uid:     memberInfo.Uid,
		})
		liberr.ErrIsNil(ctx, err)
	})
	return
}

// Cancel 取消订单
func (c *storeOrderController) Cancel(ctx context.Context, req *order.StoreOrderCancelReq) (res *order.StoreOrderCancelRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		memberInfo, orderInfo, err := service.StoreOrder().CheckOrderAuthByMemberId(ctx, req.OrderId)
		liberr.ErrIsNil(ctx, err)
		fmt.Println(orderInfo.Status)
		if orderInfo.Status == storeOrderConsts.STORE_ORDER_STATUS_CANCELED {
			liberr.ErrIsNil(ctx, errors.New("订单已关闭请勿重复操作"))
		}
		if orderInfo.Status != storeOrderConsts.STORE_ORDER_STATUS_UNPAID {
			liberr.ErrIsNil(ctx, errors.New("状态错误"))
		}
		err = service.StoreOrder().Cancel(ctx, req.OrderId)
		liberr.ErrIsNil(ctx, err)
		err = service.StoreOrderLog().Add(ctx, &do.StoreOrderLog{
			OrderId: req.OrderId,
			Content: "用户取消订单",
			LogType: storeOrderConsts.STORE_ORDER_LOG_TYPE_CANCEL,
			Uid:     memberInfo.Id,
		})
		liberr.ErrIsNil(ctx, err, "记录订单日志失败")
	})
	return
}

// Pay 支付订单
func (c *storeOrderController) Pay(ctx context.Context, req *order.StoreOrderPayReq) (res *order.StoreOrderPayRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		memberInfo := ucenterService.Context().GetLoginMember(ctx)
		wechatPayType := 0 //jssdk支付
		if req.Openid == "" {
			req.Openid = memberInfo.WxOpenid
			wechatPayType = 1 //小程序支付
		}

		_, orderInfo, err := service.StoreOrder().CheckOrderAuthByMemberId(ctx, req.OrderId)
		liberr.ErrIsNil(ctx, err)
		if orderInfo.Status != storeOrderConsts.STORE_ORDER_STATUS_UNPAID {
			liberr.ErrIsNil(ctx, errors.New("订单信息无效"))
		}
		res = &order.StoreOrderPayRes{PayType: req.PayType}
		res.Result, err = service.StoreOrder().Pay(ctx, &model.StoreOrderPayArgs{
			Uid:           ucenterService.Context().GetMemberId(ctx),
			OrderId:       req.OrderId,
			PayType:       req.PayType,
			Openid:        req.Openid,
			WechatPayType: wechatPayType,
		})
		liberr.ErrIsNil(ctx, err)
	})
	return
}

// Confirm 确认订单
func (c *storeOrderController) Confirm(ctx context.Context, req *order.StoreOrderConfirmReq) (res *order.StoreOrderConfirmRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		res = new(order.StoreOrderConfirmRes)
		uid := ucenterService.Context().GetMemberId(ctx)
		memberInfo, err := storeMemberService.StoreMember().GetByUid(ctx, uid)
		liberr.ErrIsNil(ctx, err)
		res.StoreOrderConfirmRes, err = service.StoreOrder().Confirm(ctx, memberInfo, req.StoreOrderConfirmReq)
		liberr.ErrIsNil(ctx, err)
	})
	return
}
